; LEVELS.ASM
; Handles setting up levels

;==============================================================
; START UP, CLEAR VARS, GET INTO A LEVEL
;==============================================================

init_level:

;==============================================================
; DISPLAY THE ZONE INFO
;==============================================================

	call disp_zone_info


;==============================================================
; SWITCH OFF THE SCREEN
;==============================================================

	call screen_off

;==============================================================
; RESET SOME STUFF
;==============================================================

	xor a
	ld (cur_scroll_index),a
	ld (looping_end),a
	ld (shot_eyes),a

	ld bc,144+(128*256)+20
	ld (player_y),bc

;==============================================================
; SET UP POINTERS TO LEVEL ENDS
;==============================================================

	ld hl,map_platform_exact
	ld (cur_map_pos),hl

;==============================================================
; GET POINTER TO THE LEVEL INFO TABLE FOR THIS LEVEL
;==============================================================


	ld a,(level_num)
	ld l,a
	ld h,0
	add hl,hl
	add hl,hl
	add hl,hl
	ld de,level_index
	add hl,de
	ld (level_data_table),hl
	push hl
	pop ix


;==============================================================
; SET UP LEVEL START/END POINTS
;==============================================================

	ld a,(ix+0)
	ld (cur_map_number),a
	and %01111111
	ld l,a
	ld h,0
	add hl,hl
	add hl,hl
	ld de,map_pointers-4
	add hl,de
	push hl
	call ld_hl_hl
	ld (real_level),hl
	pop hl
	inc hl
	inc hl
	call ld_hl_hl
	ld (cur_map_end),hl

;==============================================================
; SET UP WHICH ATTACK LIST TO USE
;==============================================================

	ld a,(ix+3)
	and %01111111
	ld (max_mines),a
	ld a,(ix+3)
	ld b,7
-:	srl a
	djnz -

	ld (attack_mode),a
	or a
	jr nz,+	; = 1, therefore RANDOM

	ld h,(ix+7)
	ld l,(ix+6)
	ld (attack_loc),hl
	ld (attack_base),hl
	ld a,(hl)
	push hl
	call add_enemy
	pop hl
	inc hl
	ld a,(hl)
	ld (attack_timeout),a
	inc hl
	ld a,(hl)
	dec a	; Already been added in add_enemy a few lines up, so discount it
	ld (attack_count),a

	jr ++

+:	ld a,(ix+4)
	ld (attack_timeout),a
	ld (attack_base),a

++:

	ld a,(ix+5)
	and %00001111
	ld (enemy_speed),a
	ld a,(ix+5)
	srl a
	srl a
	srl a
	srl a
	ld (land_speed),a


;==============================================================
; RESET ALL LEVEL GRAPHICS/ENEMIES/WHATEVERS
;==============================================================

	call clear_out_level
	call reset_level_graphics


;==============================================================
; CLEAR NAME TABLE
;==============================================================

	ld hl,vram_names
	call vid_goto_address
	ld bc,64*28

-:	xor a
	out (vdp_data),a
	dec bc
	ld a,b
	or c
	jr nz,-


;==============================================================
; RESET SCORE Y
;==============================================================

	call vid_destroy_sprites
	call reset_score_y
	call draw_sprites_and_score
	

;==============================================================
; SWITCH ON THE DISPLAY
;==============================================================

	call screen_on

;==============================================================
; SCROLL IN A SCREENSWORTH OF TILES [FAST!]
;==============================================================

	ld b,21
scroll_one_screen_of_level:
	push bc
	call vid_wait_vblank

	call reset_sprites

	ld bc,(player_y)
	dec c
	ld (player_y),bc
	ld d,0
	call draw_large_sprite


.ifndef DISABLE_STARS
	call draw_stars
.endif

	call finish_sprites


	call vid_wait_vblank
	call draw_sprites_and_score

	ld b,8
-:	push bc
	call scroll_one_line
	pop bc
	djnz -

	call update_stars
	call update_stars

	pop bc
	djnz scroll_one_screen_of_level

	; Load level music
	ld hl,test_music
	call load_music

	ret

;==============================================================
; END OF MAIN LEVEL LOADER
;==============================================================


;==============================================================
; RESET THE LEVEL'S GRAPHICS
;==============================================================

reset_level_graphics:

	ld ix,(level_data_table)


;==============================================================
; CLEAR ALL SPRITES
;==============================================================

	call reset_sprites
	call finish_sprites
	call display_sprites


;==============================================================
; LOAD THE MAIN TILES AND SPRITES
;==============================================================


	ld hl,grfx_sprites
	ld de,grfx_sprites_end-grfx_sprites
	ld a,2
	ld bc,256
	call vid_copy_tileset

	ld hl,grfx_tiles
	ld de,grfx_tiles_end-grfx_tiles
	ld a,2
	ld bc,0
	call vid_copy_tileset

;==============================================================
; LOAD SCORE DIGITS
;==============================================================
	
	ld hl,$14B*32
	call vid_goto_address

	ld hl,beeb_font+8*('0'-$20)
	ld b,10
--:	push bc
	ld b,8
-:	ld a,(hl)
	out (vdp_data),a
	xor a
	out (vdp_data),a
	out (vdp_data),a
	;ld a,(hl)
	;srl a
	;xor (hl)
	out (vdp_data),a
	inc hl
	djnz -
	ld b,8*4
-:	out (vdp_data),a
	djnz -
	pop bc
	djnz --


;==============================================================
; LOAD GAME PALETTE
;==============================================================

	ld a,(ix+1)
	and %00001111
	ld l,a
	ld h,0
	add hl,hl
	ld d,h
	ld e,l
	add hl,hl
	add hl,de
	ld de,game_palettes
	add hl,de
	xor a
	call vid_goto_palette
	xor a
	out (vdp_data),a
	out (vdp_data),a
	ld a,2
	ld b,6
	call vid_load_palette


;==============================================================
; LOAD SPRITE PALETTE
;==============================================================

	ld a,(ix+1)
	srl a
	srl a
	srl a
	srl a
	ld l,a
	ld h,0
	add hl,hl
	ld d,h
	ld e,l
	add hl,hl
	add hl,de
	ld de,sprite_palettes
	add hl,de
	ld a,32
	ld b,8
	call vid_load_palette


;==============================================================
; HOT-SWAP TILES
;==============================================================

	ld a,(ix+2)
	srl a
	srl a
	srl a
	srl a
	or a
	jr z,+
	ld l,a
	ld h,0
	add hl,hl
	ld de,swap_pointers-2
	add hl,de
	call ld_hl_hl

	push hl
	pop ix
	
-:	ld a,(ix+0)
	or a
	jr z,+

	ld l,(ix+1)
	ld h,0
	add hl,hl
	add hl,hl
	add hl,hl
	add hl,hl

	ld de,grfx_tiles
	add hl,de
	ld de,16
	ld a,2
	ld b,0
	ld c,(ix+0)
	call vid_copy_tileset

	inc ix
	inc ix
	jr -

+:
	xor a 
	ld (death_timer),a

;==============================================================
; ADD THE STAR TILE
;==============================================================

	
	ld hl,32*$148
	call vid_goto_address

	ld b,2*4
	ld a,%0000011
-:	out (vdp_data),a
	djnz -
	
	ld a,31
	call vid_goto_palette
	ld a,%11111111
	out (vdp_data),a
	ld a,%00001111
	out (vdp_data),a

	ret


;==============================================================
; CLEAR OUT LEVEL (MINES, ENEMIES, ETC)
;==============================================================

clear_out_level:


	ld hl,bullets
	xor a
	ld b,16
	call mem_set

	ld hl,explosions_sprites
	xor a
	ld b,max_explosions_sprites*4
	call mem_set

	ld hl,explosions_terrain
	xor a
	ld b,max_explosions_terrain*4
	call mem_set

	ld hl,mines_table
	xor a
	ld b,8*4
	call mem_set

	ld hl,enemies_table
	xor a
	ld b,8*8
	call mem_set

	ret