; SOUND.ASM
; Sound effects player/handler

;==============================================================
; CLEAR & UPDATE ALL SOUND EFFECTS
;==============================================================


clear_sound:
	; Clear ALL sound
	xor a
	ld b,80
	ld hl,sound_effects
	call mem_set

;==============================================================
; UPDATE [OUTPUT] ALL SOUNDS IN BUFFER
;==============================================================

update_sound:

	ld ix,sound_effects

	xor a
	ld (sound_slot_num),a
.ifdef FAVOUR_VOLUME
	ld a,$FF
.endif
	ld (sound_r_priority),a

	ld a,%11011111
	ld (sound_r_vol),a

play_sound_slot:


	ld h,(ix+1)
	ld l,(ix+0)
	ld a,l
	or h
	jp z,next_sound_slot

	ld a,(ix+2)
	dec a
	ld (ix+2),a
	inc a
	or a
	jr nz,just_update_sound



sound_next_section:
	ld h,(ix+1)
	ld l,(ix+0)
	ld a,(hl)
	or a
	jp z,sound_has_finished

	; Sound has NOT finished!
	; First, set the VOLUME
	ld a,(hl)
	and %00111100	; Volume info
	add a,a
	add a,a
	ld (ix+3),a	; Save away!

	ld a,(hl)
	and %00000011	; Top 2 bits of frequency
	ld b,a
	inc hl
	ld a,(hl)
	ld c,a
	ld (ix+6),b
	ld (ix+5),c

	inc hl
	ld a,(hl)
	ld (ix+2),a
	inc hl
	ld a,(hl)
	ld (ix+9),a

	inc hl
	ld a,(hl)
	ld b,a
	
	inc hl
	ld (ix+1),h	; Save pointer to next sound info chunk
	ld (ix+0),l

	ld hl,0

	or a
	jr z,no_mod_freq
	ld de,1

	and %10000000
	jr z,no_mod_freq_down

	ld de,-1
	ld a,b
	neg
	ld b,a

no_mod_freq_down:

-:	add hl,de
	djnz -

no_mod_freq:

	ld (ix+8),h
	ld (ix+7),l

just_update_sound:

	; We've got this far, so let's UPDATE THE PSG:


	ld h,(ix+6)
	ld l,(ix+5)
	ld a,l
	and %00001111
	or  %11000000
	ld e,a

;	out (psg_port),a

	ld a,h
	add a,a
	add a,a
	add a,a
	add a,a
	ld h,a
	ld a,l
	srl a
	srl a
	srl a
	srl a
	or h
	and %00111111
	ld d,a

;	out (psg_port),a


	; DE = Pitch value to output to PSG

	push de
	ld h,(ix+6)
	ld l,(ix+5)
	ld d,(ix+8)
	ld e,(ix+7)
	add hl,de
	ld (ix+6),h
	ld (ix+5),l
	pop de
	
	; Now set the output volume:

	ld a,(ix+3)
	srl a
	srl a
	srl a
	srl a
	or %11010000
	ld h,a
;	out (psg_port),a

	ld a,(ix+3)
	ld b,a
	ld a,(ix+9)
	add a,b
	ld (ix+3),a


	; AT THIS POINT
	; DE = pitch to output bytes
	;  H = volume to output byte


	ld a,(sound_r_priority)
	ld b,a
.ifdef FAVOUR_VOLUME
	ld a,(ix+3)
	cp b
	jr nc,+
.else
	ld a,(ix+4)
	cp b
	jr c,+
.endif


	ld (sound_r_priority),a
	ld a,h
	ld (sound_r_vol),a
	ld (sound_r_pitch),de

+:

	jr next_sound_slot


sound_has_finished:
	;ld a,%11011111
	;out (psg_port),a

	xor a
	ld (ix+0),a
	ld (ix+1),a
	ld (ix+4),a

	ld (ix+5),a
	ld (ix+6),a

	ld a,$FF
	ld (ix+3),a


next_sound_slot:
	call add_ix_10
	ld a,(sound_slot_num)
	inc a
	cp 8
	jr z,write_to_psg
	ld (sound_slot_num),a
	jp play_sound_slot

write_to_psg:

	ld a,(sound_r_priority)

.ifdef FAVOUR_VOLUME
	cp $FF
.else
	or a
.endif

	jr nz,+

	ld a,%11011111
	out (psg_port),a
	ret

+:


	ld a,(sound_r_pitch)
	out (psg_port),a

	ld a,(sound_r_pitch+1)
	out (psg_port),a

	ld a,(sound_r_vol)
	out (psg_port),a

	ret


add_ix_10:
	call inc_ix_4
	call inc_ix_4
	inc ix
	inc ix
	ret

;==============================================================
; ADD A PARTICULAR SOUND EFFECT TO THE SOUNDS TABLE
;==============================================================

play_sound:
	ld l,a
	ld ix,sound_effects
	ld b,8
-:	ld a,(ix+0)
	or (ix+1)
	jr z,+
	call add_ix_10
	djnz -
	ret

+:	; Found a free sound slot

	ld a,l

	ld b,a
	add a,a
	add a,b
	ld l,a
	ld h,0
	ld de,sound_effect_table
	add hl,de

	ld a,(hl)
	ld (ix+4),a
	inc hl
	call ld_hl_hl
	ld (ix+1),h
	ld (ix+0),l
	ld a,1
	ld (ix+2),a
	ret